﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace Logofatu
{
    public partial class P12Koch : Form
    {
        private static double Sqrt3 = Math.Sqrt(3.0);
        private static double Sqrt3Div6 = Math.Sqrt(3.0) / 6.0;       

        private int iterations;

        public P12Koch(int numIterations)
        {
            InitializeComponent();
            this.iterations = numIterations;
        }

        public void paint(Graphics g)
        {
            Size rect = panel1.Size;
            int diff = rect.Width - rect.Height;
            PaintInvariant paintInv = new PaintInvariant(g, 5);
            int l;
            if ( diff < 0 )
            {
                diff *= -1;
                l = rect.Width / 2;
                paintInv.dx = 0;
                paintInv.dy = diff / 2;
            }
            else
            {
                l = rect.Height / 2;
                paintInv.dx = diff / 2;
                paintInv.dy = 0;
            }
            double xa, ya, xb, yb, t = l;
            xa = l;
            ya = l - t;
            xb = l - Sqrt3 * t / 2;
            yb = l + t / 2;
            drawKoch(paintInv, this.iterations, (int)xa, (int)ya, (int)xb, (int)yb);
            xa = l + Sqrt3 * t / 2;
            ya = l + t / 2;
            drawKoch(paintInv, this.iterations, (int)xb, (int)yb, (int)xa, (int)ya);
            xb = l;
            yb = l - t;
            drawKoch(paintInv, this.iterations, (int)xa, (int)ya, (int)xb, (int)yb);
        }

        private void drawKoch(PaintInvariant paintInv, int n, double xa, double ya, double xb, double yb)
        {
            if ( n > 1 )
            {
                double xc, yc, xm, ym, xd, yd;
                xm = (xa + xb) / 2 + Sqrt3Div6 * (ya - yb);
                ym = (ya + yb) / 2 + Sqrt3Div6 * (xb - xa);
                xc = (2 * xa + xb) / 3;
                yc = (2 * ya + yb) / 3;
                xd = (xa + 2 * xb) / 3;
                yd = (ya + 2 * yb) / 3;
                drawKoch(paintInv, n - 1, xa, ya, xc, yc);
                drawKoch(paintInv, n - 1, xc, yc, xm, ym);
                drawKoch(paintInv, n - 1, xm, ym, xd, yd);
                drawKoch(paintInv, n - 1, xd, yd, xb, yb);
            }
            else
            {
                paintInv.g.DrawLine(Pens.Black, (int)xa, (int)ya, (int)xb, (int)yb);
            }
        }        

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            paint(e.Graphics);
        }

        private void panel1_Resize(object sender, EventArgs e)
        {
            this.Refresh();
        }
    }

    class PaintInvariant
    {
        public Graphics g;
        public int dx, dy, minL;

        public PaintInvariant(Graphics g, int minL)
        {
            this.g = g;
            this.minL = minL;
        }
    }

}
